<?php
namespace Macro;
/**
 * HideEmail - macro class
 * 
 * Macro to obfuscate mailto link in a-href to prevent spammer
 * 
 * @copyright 2020 ABC Web Design Agency
 *
 * @license MIT
 *
 * @package hideemail
 * @version 1.0
 * @author  ABC Web Design Agency <info@mycompany.com>
 * @link    https://www.mycompany.com
 */

class HideEmail extends \SCHLIX\cmsMacro {

    /**
     * Text to display
     * @var string 
     */
    protected $str_text_to_display;
    /**
     * Unused for this example, just ignore
     * @var mixed
     */
    protected static $has_this_macro_been_called;
    
    /**
     * Logic part 1 - replace all mailto links
     * @param string $html
     * @return string
     */
    private function replaceAllMailtoLinks($html)
    {
        // mission: 
        // 1) search for all <a href="mailto:*">innerhtml</a>
        // 2) replace it with obfuscated javascript link
        // 3) when viewed by spider bots/spam harvesters, it will be just
        //    something like <span id="e1234567">[xxxxxxxxxxxxx]</span>
        preg_match_all ("/<a href=\"mailto\:([^\"]+)\"[^>]*>([^<]*)<\/a>/",$html,$matches);
        // if the regular expression yields matches
        if (___c($matches) > 0)
        {
            $count = ___c($matches[0]);
            for($i = 0; $i < $count; $i++)
            {            
                // the full <a href"mailto:*">innerhtml</a>
                $full_tag_to_replace = $matches[0][$i];    
                // the email address part
                $ahref_mailto =  $matches[1][$i];
                // the innerhtml
                $ahref_inner_html =  $matches[2][$i];
                if (filter_var($ahref_mailto, FILTER_VALIDATE_EMAIL) !== FALSE)
                {
                     $replacement = $this->hideEmailAddress($ahref_mailto, $ahref_inner_html);
                     $html = str_replace($full_tag_to_replace, $replacement, $html);
                }
            }
        }
        return $html;
    }
    

    /**
     * Logic part 2 - convert email address to Javascript link
     * Source code copied and pasted from: http://www.maurits.vdschee.nl/php_hide_email/
     * with slight modification (added $inner_html)
     * @param string $mailto_link
     * @param string $inner_html
     * @return string
     */
    private function hideEmailAddress($mailto_link, $inner_html)
    { 
      // the inner part of the link, which could be an email address
      // or it could be just a text. 
      // if it's an email address, let's replace it as well
      if (filter_var($inner_html, FILTER_VALIDATE_EMAIL) !== FALSE)
      {
         list($name, $domain) = explode('@', $inner_html);
         $inner_html = $name.' [at] '.$domain;         
      }
      // end modification of http://www.maurits.vdschee.nl/php_hide_email/
      $character_set = '+-.0123456789@ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz';
      $key = str_shuffle($character_set); $cipher_text = ''; $id = 'e'.rand(1,999999999);
      for ($i=0;$i<strlen($mailto_link);$i+=1) $cipher_text.= $key[strpos($character_set,$mailto_link[$i])];
      $script = 'var a="'.$key.'";var b=a.split("").sort().join("");var c="'.$cipher_text.'";var d="";';
      $script.= 'for(var e=0;e<c.length;e++)d+=b.charAt(a.indexOf(c.charAt(e)));';
      $script.= 'document.getElementById("'.$id.'").innerHTML="<a href=\\"mailto:"+d+"\\">'.$inner_html.'</a>"';
      $script = "eval(\"".str_replace(array("\\",'"'),array("\\\\",'\"'), $script)."\")"; 
      $script = '<script type="text/javascript">/*<![CDATA[*/'.$script.'/*]]>*/</script>';
      // always use ___h (htmlspecialchars)
      return '<span id="'.$id.'">['.___h($this->str_text_to_display).']</span>'.$script;
    }


    /*
     * Run the macro
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @param array|string $data
     * @param object $caller_object
     * @param string $caller_function
     * @param array $extra_info
     * @return bool
     */
    public function Run(&$data, $caller_object, $caller_function, $extra_info = NULL) {
        global $HTMLHeader;
        
        // If it hasn't been configured, simply display xxxxxxxxx
        $this->str_text_to_display = $this->config['str_text_to_display'];
        if (empty($this->str_text_to_display))
            $this->str_text_to_display = 'xxxxxxxxxxxxxx';
        
        // This is just an example macro
        // we're going to ignore which caller function it is
        // because it doesn't matter.
        // regardless, we want to search and replace the text
        /*
        switch ($caller_function)
        {
            case 'viewItemByID':   break;
            case 'viewCategoryByID':  break;
            case 'viewChildCategory':  break;
            case 'viewChildItem': break;
            case 'viewMainPage':  break;
            default:  break;
        }*/
	// Who's calling this?
        // only cmsApplication_List and its inherited classes have
        // database fields.
        if (is_a($caller_object,'\\SCHLIX\\cmsApplication_List'))
        {
            // look for summary and description from database field            
            // $app_name  = $caller_object->getApplicationName();
            $fields_to_replace = ['summary', 'description'];
            foreach ($fields_to_replace as $field)
            {
                if (array_key_exists($field, $data) && !empty ($data[$field]))
                {
                    $data[$field] = $this->replaceAllMailtoLinks($data[$field]);
                }
            }
        } else if (is_a($caller_object,'\\SCHLIX\\cmsBlock'))
        {
            if (is_string($data))
            {
                $data = $this->replaceAllMailtoLinks($data);
            }
            
        }
        // with &$data, the data is replaced right away
        return true;
    }

}
            